
/******************************************************************************
    SG_DTMF.c         DTMF accurate sinewave generator
    Copyright:        Open-Source, Feb 2011, written by www.RomanBlack.com
    MCU:              P18F27K42
    FOSC : 64Mhz

  This uses a zero-error DDS algorithm to generate 2 good quality sinewaves
  at the same time on one PIC PWM output, and both sinewaves have extremely
  accurate frequencies. This project uses the algorithm to generate
  2 sinewaves for telephone DTMF use, but it could also be used for
  polyphonic music generation of many exact notes with defined waveshapes.
  The sinewave table for this project has a range of 0-99 and 64 samples.
  For a full description of theory see; www.RomanBlack.com/one_sec.htm#BDA
  Note!! This uses 40x16 text mode on GLCD.
******************************************************************************/

  #define Version "2022_1206"
  //rajout etat led HT9170B  en tete de cde
 // #define Version "2022_1205"
 // #define Forum_test
  //#define Version "2022_1121"
   // rajout test PWM avec SQA analyser    
   //Q=quit test 1=pwm enable  0= pwm disabled
   // test comparatif avec  PWM1_Set_Duty(pmw1)   FMT=0 
   //  et CCPR1H=pmw1 CCP1RL=0 et FMT=1
  //#define Version "2022_1117"
   // rajout choix RC4 ou RC2 vi pin RB1   RB1=0 => RC4 sinon RC2
   //  #define Version "2022_1116"
   // aiguillage sortire PWM1 sur RC4 via PPS ..OK
   //   #define Version "2022_1114"
   // sortie sur RC2
   // new table sine64 divise en 2
   // BAD detection avec HT7190B !
   
  // #define Version "2022_1113"
   //rajout commande RC    row Colonne clavier via terminal UART
  //#define Version "2022_1109"
 // Rajout nom de touche
  // Rajout delay ou appui touche Keyboard

  //

 // Hardware
 /*
 Pin 28  RB7  --x  x----ICSP Data
 Pin 27  RB6  --x  x-----ICSP Clock
 Pin 26  RB5
 Pin 25  RB4
 Pin 24  RB3
 Pin 23  RB2
 Pin 22  RB1  <----- to show details of each wav bloc reading
 Pin 21  RB0
 Pin 20      VDD --------+5V
 Pin 19      Vss ---------Gnd          fil noir cable prolific
 Pin 18  RC7  RX UART  <-- Keyboard    fil Vert cable prolific
 Pin 17  RC6  TX UART  --> Display     fil Blanc cable prolific
 Pin 16  RC5  --> CTS

 Pin 15  RC4  --> sortie PWM1 CCP1
 Pin 14  RC3  SCL  I2C

 Pin 13  RC2  --> PWM1 CCP1 ouput --> filtre passe bas avec transistor en suiveur -> sortie basse impedance
 Pin 12  RC1
 Pin 11  RC0

 Pin 10  RA6
 Pin  9  RA7
 Pin  8  VSS ---------  Gnd
 Pin  7  RA5  ----> Led
 Pin  6  RA4  ----> led
 Pin  5  RA3  --> SQA analyser  trigger input
 Pin  4  RA2
 Pin  3  RA1
 Pin  2  RA0  <---- Analog Input
 Pin  1  RE3 MCLR  Reset <--------ICSP PGRM
                          |----x  x----/0V   cavalier RESET
 */
 #define Directory "C:\\_MikroC\\_MesProjets_MikroC\\_18F27K42_Roman_Black_DTMF_Generateur_2022"
#define Project "PIC18F27K42_Roman_Black_DTMF_Generateur_2022"
#define Source  "PIC18F27K42_Roman_Black_DTMF_Generateur_RC2_ou_RC4"
#define Others "No "
#define Eeprom "not used ...."
#define Config "P18F27K42_Fosc_Interne_Legacy_interrupts_16x4_PLL_64MHz.cfgsch"
#define PROCESSOR "18F27K42"
#define POWER_SUPPLY  " 3.3V  5V"
#define Chargeur "."

#define OSCILLATEUR_INTERNE
#define FOSC "64.0"  // MHz
#define BAUD "19200"  // UART1

#define XOFF   19
#define XON    17
#define EOT    4
#define ETX    3

#define CLS 12   // specific au Terminal VBRAY
#define CR 13    //0x0D
#define LF 10
#define TAB 9
#define BACK 8
#define Beep 7
#define Separator 0x20   // space
#define ON 0    // because led tire au +VCC
#define OFF 1
#define SPECIAL 1
#define NORMAL 0

#define  PWM_on_RC4

#ifndef Byte
#define Byte unsigned char
#endif

#ifndef Word
#define Word unsigned int
#endif
#define Separateur ','


const code char mesg0[]=" Directory :"Directory"\r\n";
const code char mesg1[]=" MikroC pro 7.60\r\n";
const code char mesg2[]=" Projet :"Project"\r\n";
const code char mesg3[]=" generation de sons DTMF via 8bits PCM output\r\n";
const code char mesg4[]=" Config bit : "Config"\r\n FOSC:"FOSC" MHz\r\n";
const code char mesg5[]=" Eeprom: "Eeprom"\r\n";
const code char mesg6[]=" Source : "Source"_"Version".c\r\n";
const code char mesg7[]=" 18F27K42 UART1 19200 bds\r\n"; // ADC1 12 bits sur EA1 \r\n";
const code char mesg8[]=" usage de l'Algorithme de Roman Black\r\n"; //Protocole RTS sur RC5 outp\r\n";
const code char mesg9[]=" PWM1 sur RC2 par defaut  si RB1=1 , UART1 : Rx RC7  TX RC6\r\n";
const code char mesg10[] ="Choix PWM1 sur RC4 si RB1=0\r\n";
const code char * Messages[]={mesg0,mesg1,mesg2,mesg3,mesg4,mesg5,mesg6,mesg7,mesg8,mesg9,mesg10};
const code char Blancs[]="  ";

sbit Led_Rouge at LATA.B4;
sbit Led_Rouge_Dir at TRISA.B4;
sbit SQA at LATA.B3  ;
sbit SQA_Dir at TRISA.B3 ;
sbit Test_timer2_CCP1_PWM at PORTA.B2;
sbit Test_timer2_CCP1_PWM_Dir at TRISA.B2;
sbit CTS at LATC.B5;
sbit CTS_Dir at TRISC.B5;
sbit Choix_RC4 at PORTB.B1;
sbit Choix_RC4_Dir at TRISB.B1;
sbit Keep_File at PORTB.B2;
sbit  Keep_File_Dir at TRISB.B2;

#define MAX_LEN1 130
#define MAX_LEN2 128
#define MAX_LEN3 128

//***************   DTMF ************************
// defines for the DTMF sine frequencies
// Sinewave is a lookup table of 64 steps, to generate 1633 Hz : each step is 1633 * 64
// 24bit Binary divider uses byte2 (/65536)
// PWM freq is 128 PIC instructions (8Mhz xtal *PLL) = 62500 Hz
// So the math is;  Fout * 64 * 65536 / Fpwm
// we can use a constant Fout * (64 * 65536 / 62500) = Fout * 67.108864
#define Frow0 46775   // 697 Hz  DTMF rows
#define Frow1 51674   // 770 Hz
#define Frow2 57177   // 852 Hz
#define Frow3 63149   // 941 Hz

#define Fcol0 81135   // 1209 Hz  DTMF columns
#define Fcol1 89657   // 1336 Hz
#define Fcol2 99120   // 1477 Hz
#define Fcol3 109589  // 1633 Hz

 #ifdef Forum_test
const char Cells[10] = {0x01, 0x30, 0x08, 0x10, 0x15, 0x13, 0x01, 0x04, 0x14, 0x60} absolute 0x7770;
const char Cells1[10] = {0x02, 0x31, 0x09, 0x11, 0x16, 0x14, 0x02, 0x05, 0x15, 0x61} absolute 0x777A;
unsigned char Cell_0=0;
#endif

 // Global Variables for DTMF
unsigned long waveA   absolute 0x20; // 32bit accumulators for the  sinewave1  DDS
unsigned char waveA_2 absolute 0x22; // overload for fast access to byte 2
unsigned long waveB   absolute 0x24; // 32bit accumulators for the  sinewave2  DDS
unsigned char waveB_2 absolute 0x26; // overload for fast access to byte 2
unsigned long BDA_periodA;  // 32bit periods for the 2 sinewaves
unsigned long BDA_periodB;
unsigned char drow;     // DTMF selected row and column
unsigned char dcol;

// This uses dual sinewaves of 28% different amplitudes, to match the spec for "twist" in
 // telephone DTMF to ensure the higher freq has a higher amplitude, to cope with line losses.
  const unsigned char sine64low[64]={
  39,42,46,50,53,57,60,63,66,68,71,72,74,75,76,77,77,77,76,75,74,72,71,68,66,63,60,57,53,50,46,42,
  39,35,31,27,24,20,17,14,11,9,6,5,3,2,1,0,0,0,1,2,3,5,6,9,11,14,17,20,24,27,31,35};
  const unsigned char sine64high[64] = {
  50,54,59,64,68,73,77,81,85,88,91,93,95,97,98,99,99,99,98,97, 95,93,91,88,85,81,77,73,68,64,59,54,
  50,45,40,35,31,26,22,18,14,11,8,6,4,2,1,0,0,0,1,2,4,6,8,11,14,18,22,26,31,35,40,45};

unsigned int Frow_value[]={  697,  770, 852, 941}; // Hz
unsigned int Fcol_value[]={ 1209,1336,1477,1633}; //hz
                         //  1     2    3    A     697
                         //  4     5    6    B     770
                         //  7     8    9    C     1477
                         //  *     0    #    D     1633
unsigned long Frow[]={ 46775 ,51674 , 57177 , 63149};
unsigned long Fcol[]={ 81135,89657,99120,109589};
//  exemp              81135/67.1088864=1209  Hz

const char Leds_HT9170B[][5]={
"LLLH","LLHL","LLHH",  // 1 2 3
"HHLH",  //A
"LHLL","LHLH","LHHL",  // 4 5 6
"HHHL",  //B
"LHHH","HLLL","HLLH",  // 7 8 9
"HHHH",  //C
"HLHL","HLHH","HHLL",  // 0 * #
"LLLL"}; //D
char Sequence[]="123A456B789C0*#D";

//=================================================




unsigned int i,j,k,l,m,n;

volatile unsigned char Temp=0;
volatile unsigned char Buffer1[MAX_LEN1];
volatile int Flag_Buffer1;
volatile unsigned char c1,c2,cx;
volatile unsigned int i1;
volatile unsigned int Index1;
volatile unsigned int CptErr;
signed char c0;
char dummy=0;
unsigned int Etape=0;

static unsigned char * p1;
static unsigned char * p2;

static unsigned char CRam1[MAX_LEN2];
unsigned char TEXTE [MAX_LEN3];
unsigned char *txt;
unsigned char tmp[10];

volatile unsigned int Count2, CountS;
volatile int Cpt1=0;
//volatile int Cpt3;
//volatile int Max_Timer1=10;
volatile int  Flag_Timer0=0;
volatile int  Flag_Timer6=0;
volatile int Flag_RB0=0;
//volatile unsigned int Max_Timer1=4;
//unsigned int SEA1,EA1,Nb;
float F0,F1;
unsigned long L1,L2;

int addr;
int error;
unsigned int Count0 = 0;
unsigned char Etat_clock=0;
int IRatio=0;

volatile signed int Cpt_Timer6=0;
unsigned char pwm1;



void CRLF1(void) ;
void Raz_Buffer1(void) ;
void UART1_Write_CText(const char *txt);
void Print(char *tx);
void CPrint(const char *tt);
void strConstRamCpy(unsigned char *dest, const code char *source);
void Arme_Timer1(signed int  Max_Timer1);
void Init_Timer6( unsigned int N); //    0.5sec  at 500Kz
void CCP1_PWM_Config(void);
void make_DTMF_tone(void);
void Init_Hardware(void) ;


void Interrupts() iv 0x0008 ics ICS_AUTO
{  //signed  int k1;

  //------  RX UART IT  -------------------------------------------
   if (( U1RXIF_bit==1) && ( U1RXIE_bit==1))  //RC1IE_bit==1) )
   {
      // U1TXB=='?';
      c1 = UART1_Read();
      if (U1ABDOVF_bit==1)    // voir parag  31.2.2.4   page 479
      {
        U1ABDOVF_bit = 0 ;
        CptErr++;
        c1=U1RXB ;
       }
       if(U1FERIF_bit==1 )
       {
         CptErr++;
         c1 = UART1_Read();
       }
       //REGISTER 31-4: UxERRIR: UART ERROR INTERRUPT FLAG REGISTER
        if(U1RXBKIF_bit==1 )
       {
         CptErr++;
         c1 = UART1_Read();
       }

         Buffer1[i1] = c1;
         // special fichier wave :
         Index1++;
         i1++;
         if ( (c1==13)|| (i1>4) )
          {
              // CTS=1;
                U1RXIE_bit=0;
                Flag_Buffer1=1;
                i1=0;
          }
      }
  if( ( PIE9.TMR6IE==1) && ( PIR9.TMR6IF==1) )
  {
    if ( Cpt_Timer6>0)Cpt_Timer6--;
    if ( Cpt_Timer6<1)
    { Flag_Timer6=1;
      PIE9.TMR6IE=0;
      T6CON.T6ON=0;
      }
    PIR9.TMR6IF=0;
   }
}

// --- Copie le texte depuis FLASH ROM vers RAM
void strConstRamCpy(unsigned char *dest, const code char *source)
 {
  while (*source)*dest++ = *source++ ;
  *dest = 0 ;    // terminateur
}

void CRLF1()
{
 UART1_Write(CR); UART1_Write(LF);
}


void Print(char *tx)
 {
   while (*(tx)>0)
   UART1_Write(*(tx++));
 }

 void CPrint(const char *tt)
 {
   while (*(tt)>0)
   UART1_Write(*(tt++));
 }

void UART1_Write_CText(const char *txt)
 {
   while (*(txt)>0)
      UART1_Write(*(txt++));
 }

void Raz_Buffer1()
{    int i;
       //  CTS=1;
        // nettoye le debut de buffer ,car utilis pour init BT
        for(i=0;i<MAX_LEN1-1;i++) Buffer1[i]=0;
        Buffer1[0]=0;
        i1=0;
        p1=0;
        Index1=0;
        Flag_Buffer1 =0 ;
        U1RXEN_bit=1;
         U1ERRIR = 0x00;
         U1ERRIE = 0x00;
       //  if (U1RXIF_bit==1) c1=UART1_Read();
        Flag_Buffer1=0;
         c1=0;
        U1RXIE_bit=1;
      //  CTS=0;
        GIE_bit=1;
}

void Init_Hardware()
{
  PMD0 = 0x00;
  PMD1 = 0x00;
  PMD2 = 0x00;
  PORTA = 0x00;
  ANSELA=0x02;  // RA1 analog
  TRISA = 0b10100111 ;   // RA0 ..RA2 as input    RA3,RA4,RA6 as output , RA5 & RA7 as  input
  Test_timer2_CCP1_PWM_Dir =1; //RA2
  SQA_Dir=0;  // RA3
  SLRCONA.B6=0; // clockout 16Mhz
  SQA=0;  // RA3 output
  WPUA=0xFC;

  ANSELB=0;
  PORTB = 0x00;
  TRISB = 0xFF;    //  B0..B7 Output   ATTENTION B7 B6 for ICSP Pickit3
  WPUB=0xFF;
  CM1CON0=0;
  CM2CON0=0;
  Choix_RC4_Dir=1; //RB1;
  Keep_File_Dir= 1; // RB2;

  PMD0 = 0x00;
  PORTC=0;
  ANSELC=0 ;
  TRISC = 0b10000000;    //
 // #ifdef With_RTC_DS3231
 //  RTC_Forcee_Dir=1;
 //  #endif
  CTS_Dir=0;  // RC5
  CTS=0;
  SQA_Dir=0;
  SQA=0;
  FVRCON=0;
  FVRCON.FVREN=1;  // page 597  paragraph 34.3
  FVRCON.ADFVR1=1; // 01 = Fixed Voltage Reference Peripheral output is 1,024)
  FVRCON.ADFVR0=0;
  SLRCONC=0x00;
}


void CCP1_PWM_Config(void)
 {
    CCP1CON=0;
    CCP1CON.CCP1FMT=0;  // 1= left alignement 0=right alignment
    CCP1CON.CCP1MODE3=1;   //PWM 0b1100';        //   table 22-1  p322
    CCP1CON.CCP1MODE2=1;
    CCP1CON.CCP1MODE1=0;
    CCP1CON.CCP1MODE0=0;
    CCPTMRS0.C1TSEL1=0;
    CCPTMRS0.C1TSEL0=1;
    CCP1CAP=0;
    CCP1CON.B7=1;// module enable
}
  /*

void PWM1_Load_Duty_Value(unsigned int Duty_Value)
 {
// suivant config FORMAT   voir FIGURE 23-5: PWM 10-BIT ALIGNMENT  p349
// right alignement 0
  CCPR1L  = (Duty_Value & 0x00FF);      //   Writing  LSB of PWM duty cycle
  CCPR1H = 0; //  Writing 2 bits MSB of PWM duty cycle
   //left alignement   CCP1CON.CCP1FMT=1;
 // CCPR1L  = 0;
 //  CCPR1H =( Duty_Value & 0x00FF) ; //  Writing 2 bits MSB of PWM duty cycle
 }
 */


void Init_Timer6( unsigned int N) //    0.5sec  at 500Kz
{
  T6CON    = 0x00;
   // 5 => div =1/64
  T6CKPS2_bit=1;
  T6CKPS1_bit=1;
  T6CKPS0_bit=0;
   // 145 => div= 1/15
  T6OUTPS3_bit=1;
  T6OUTPS2_bit=1;
  T6OUTPS1_bit=1;
  T6OUTPS0_bit=0;
  // 4096  *64*15*255 = 500 mS avec T6CLK=5   500Khz
  T6TMR=0;
  T6CLK=0x05; // 5=500Khz
  T6PR=255 ;
  T6RST=0x00;
  T6HLT=0x04;   // Period Pulse with  Hardware Reset
  Flag_Timer6=0;
  TMR6IF_bit=0;    //PIR9.TMR6IF
  if (N>0)
  {
   Cpt_Timer6=N;
    PIE9.TMR6IE=1;
    }
    else  PIE9.TMR6IE=0;
  T6CON.T6ON=1;
}

void make_DTMF_tone(void)
{
  //-------------------------------------------------------
  // this generates the 2 accurate sinewaves together.
  // a DDS algorithm calculation is done once every PWM cycle,
  // then the 2 sinewaves are generated by the PIC PWM module.
  // For a full description of algorithm; www.RomanBlack.com/one_sec.htm#BDA
  // NOTE! for good speed PWM TMR2 is manually polled, not an interrupt.
  //-------------------------------------------------------
   unsigned char pwm;
  // loop and generate dual sinewave DTMF tone
  TMR6IE_bit=0;
  CCPR1L=0;
  Init_Timer6(0);
  CCP1EN_bit=1;
   while(TMR6IF_bit==0)
   {
    while(TMR2IF_bit==0);  // wait until PWM cycle starts (sync to PWM)
    TMR2IF_bit = 0;
    // calc the A sinewave,
    waveA += BDA_periodA;                // zero-error Accumulation
    pwm = sine64low[waveA_2 & 0x3F];        // Binary Divide output (/65536) and keep 6 bits
    if(drow == 4) pwm = 0;        // disable if row is OFF
    // calc the B sinewave, and ADD the 2 waves together
    waveB += BDA_periodB;
    pwm += sine64high[waveB_2 & 0x3F];  // only enable if col is ON
    // scale the summed sinewaves and place in PWM module
    pwm = (pwm >> 1);   // scale 0-200 back to 0-100 for PWM
    pwm += 14;          // offset to centre waveform (100 into centre of 128)
    PWM1_Set_Duty(pwm);
  }
  CCP1EN_bit=0;
}

void main()
 {
   // 64 MHz
    OSCCON1 = 0x60;   // NOSC HFINTOSC; NDIV 1;
   // while(OSCCON2 != OSCCON1) //Wait for Clock Switch to occour
    OSCCON3 = 0x00;   // CSWHOLD may proceed; SOSCPWR Low power;
    OSCEN   = 0x00;   // MFOEN , LFOEN , ADOEN ,SOSCEN, EXTOEN,HFOEN all disabled;
    OSCFRQ  = 0x08;   // HFFRQ 0x08=64MHz ; 0x07=48Mhz; 0x06=32Mhz; 0x05=16Mhz ; 0x02=4Mhz
    OSCTUNE = 0;   // TUN 0;
    Etat_clock=OSCSTAT;

    Init_Hardware();
    Count0=0;
    Cpt_Timer6=0;

    for (i=0;i<8;i++)
    {
       _asm btg LATA,4 ;
       Delay_ms(250);
    }
    for (i=0;i<MAX_LEN2;i++) CRam1[i]=0;
    Led_Rouge=0; //LATA.B4=1;
    for (i=0;i<MAX_LEN3-1;i++) TEXTE[i]=0;
    for (i=0;i<MAX_LEN1-1;i++) Buffer1[i]=0;
    
    txt=&TEXTE[0];

   Unlock_IOLOCK()  ;
   RC5PPS=0;
   RC4PPS=0;
   //RC4PPS=0x09;     // PWM output sur RC4
   RC6PPS = 0x13;   //RC6->UART1:TX1;
   U1RXPPS = 0x17;   //RC7->UART1:RX1;
   Lock_IOLOCK() ;

   UART1_Init(115200);  // pas assez  fiable !!!!
   UART_Set_Active(&UART1_Read, &UART1_Write, &UART1_Data_Ready, &UART1_Tx_Idle); // set UART1 active
   Delay_ms(1000);

    UART1_Write(CLS);
    CPrint("\r\n Presentation :\r\n");
    for (i=0;i<10;i++)   CPrint(Messages[i]);
  // -------------------------------------------------------
  #ifdef Forum_test
   CPrint( "\r\n Test Forum MikroC :\r\n");
   CPrint("\r\n https://forum.mikroe.com/viewtopic.php?f=88&t=79656 \r\n");
    for (i=0;i<10;i++)
    {
       ByteToStr(  Cells[i],CRam1);
      CPrint(" Cells ");
      UART1_Write(i+48);  UART1_Write('>');
      Print(CRam1);
      CRLF1();
     }
    CRLF1();
  /*
      for (i=0;i<10;i++)
    {
      ByteToStr(  Cells1[i],CRam1);
      CPrint(" Cells1 ");
      UART1_Write(i+48);  UART1_Write('>');
      Print(CRam1);
      CRLF1();
     }
     */
      CRLF1();
       ShortToStr(Bcd2Dec(Cells[0]), txt);
       Print(txt); CRLF1();
       // oled_write(1, 0, txt, 5, 'w');
       //But it Does not work, when I assign Var = Cells and use Var in further.
       Cell_0 = Bcd2Dec(Cells[0]);
       ShortToStr(Cell_0, txt);
       Print(txt); CRLF1();
       // oled_write(1, 0, txt, 5, 'w');
       Cell_0 = Bcd2Dec(Cells[1]);
       ShortToStr(Cell_0, txt);
       Print(txt); CRLF1();
       Cell_0 = Bcd2Dec(Cells[2]);
       ShortToStr(Cell_0, txt);
       Print(txt); CRLF1();
      while(1);
    #endif
      // --------------------------------------------------
   CRLF1();
   CPrint(" Sinewave DTMF generator\r\n");
   CPrint(" Select the 2 DTMF tones\r\n");
   CPrint(" Row :   697    770    852    941    OFF\r\n");
   CPrint(" Col :  1209   1336   1477   1633    OFF\r\n");
   CRLF1();

    CCP1_PWM_Config();
    PWM1_Init(62500);   // sur RC2 par defaut !
    
    //inhibition
    CCP1EN_bit=0;
    T2CON.T2ON=0;
    CPrint(" Init PWM1 at 62500Hz ");
    if( Choix_RC4==0)   // via pin RB1
    {
    CPrint(" Choix RB1=0 => PWM  sur sortie  RC4 \r\n");
    Unlock_IOLOCK()  ;
    RC2PPS=0;
    RC4PPS=0x09;     // PWM output sur RC4
    Lock_IOLOCK() ;
    }
    else
    {
     CPrint(" Choix RB1=1 =>PWM sur sortie RC2(par defaut!)\r\n");
     Unlock_IOLOCK()  ;
     RC4PPS=0;
     RC2PPS=0x09;
     Lock_IOLOCK() ;
     }
     // reactivation
    CCP1EN_bit=1;
    T2CON.T2ON=1;
    CCP1FMT_bit=1;
    CPrint(" FIGURE 23-5: PWM 10-BIT ALIGNMENT page 349 \r\n Etat bit FMT = ");
    UART1_Write(48+CCP1FMT_bit);
    if ( CCP1FMT_bit==0)
    CPrint( " Alignement  droite ");
    else
    CPrint( " Alignement  gauche ");
    CRLF1();
    j=0;
 //  do
 //  {
     CPrint(" Test CCP1 en mode PWM associ  TMR2, pwm 16  224 \r\n");
     for (pwm1=16;pwm1<240;pwm1=pwm1+16)
     {
       ByteToStr(pwm1,CRam1);
       CPrint(" pwm1 = ");
       Print(CRam1);CRLF1();
       //PWM1_Set_Duty(pwm1);   // sur 8 bits !!!!
       // avec  FMT=0   CCPR1H=0; et   CCPR1L=pwm1;  maxima possible = 22% pour pwm1=224
       // avec FMT=1
       CCPR1L=0;
       CCPR1H=pwm1;
       SQA=1;
       Delay_ms(1);
       SQA=0;
     }
   /*
     CPrint(" saisir Q pour quitter le test\r\n");
     Raz_Buffer1();
     Delay_ms(2000);
     if (*(Buffer1)=='0')
     {
         CPrint(" Disable CCP1 \r\n");
         CCP1EN_bit=0;
     }
     if (*(Buffer1)=='1')
     {
        CPrint(" Enable CCP1 \r\n");
        CCP1EN_bit=1;
      }
    }
   while(*(Buffer1)!='Q');
   CPrint(" ..fin de test\r\n");
   */
   CRLF1();
   
    CPrint(" Init Timer6  3 sec : ");
    Init_Timer6(6);
    TMR6IE_bit=1;
    GIE_bit=1;
    while(Flag_Timer6==0)
    {
     UART1_Write('.');
     Delay_ms(250);
    }
    CRLF1();
    
    CCP1FMT_bit=0;
    CPrint(" Remise FMT bit   Zero !!! \r\n");
    CPrint("     pour pouvoir reutiliser la fonction MikroC (en 8 bits)\r\n");

    CPrint(" default starting values for the DTMF drow=0 and dcol=0\r\n");
    drow = 0;
    BDA_periodA = Frow0;
    dcol = 0;
    BDA_periodB = Fcol0;
    CPrint(" F1=");
    WordToStr(Frow_value[drow],CRam1);
    Print(CRam1);
    CPrint(" Hz , F2=");
    WordToStr(Fcol_value[dcol],CRam1);
    Print(CRam1);
    CPrint(" Hz\r\n");
    SQA=1;
    make_DTMF_tone();
    SQA=0;
    Delay_ms(2000);
    CRLF1();
   // goto ICI;
   
    CPrint(" Generation des 16 codes DTMF  intervalle 1sec\r\n");
    for (drow=0;drow<4;drow++)
    {
       for (dcol=0;dcol<4;dcol++)
       { 

           i=drow*4+dcol;
           UART1_Write(' ');
           UART1_Write(*(Sequence+i));
           UART1_Write(' ');
           CPrint(Leds_HT9170B[i]);
           UART1_Write(TAB);
           CPrint(" F1=");
           WordToStr(Frow_value[drow],CRam1);
           Print(CRam1);
           CPrint(" Hz , F2=");
           WordToStr(Fcol_value[dcol],CRam1);
           Print(CRam1);
           CPrint(" Hz   PA=");
           BDA_periodA = Frow[drow];
           LongWordToStr( BDA_periodA,CRam1);
           Print(CRam1); CPrint("  PB=");
           BDA_periodB = Fcol[dcol];
           LongWordToStr( BDA_periodB,CRam1);
           Print(CRam1); CRLF1();
           make_DTMF_tone();
         Delay_ms(1000);
        }
      }
    ICI:
     CRLF1();
     CPrint(" OSCTUNE ajustable via cde Oxx   avec xx 0  62 \r\n");
     CPrint(" code touche 1 par defaut\r\n");
     CPrint(" modif code via saisie de RxCx  (Range et Colonne de touche)\r\n");
     CRLF1();
     drow=0;
     dcol=0;
     BDA_periodA = Frow[drow];
     BDA_periodB = Fcol[dcol];
     make_DTMF_tone();

     do
     {  
         Raz_Buffer1();
         Delay_ms(1000);
         if ( Flag_Buffer1==1)
         {
             if(   *(Buffer1)=='O'  )
             {
               cx=*(Buffer1+1);
               c0=cx-48;
               c0= c0*10;
               cx= *(Buffer1+2);
               c0=c0 + ( cx -48 );
               if (c0 <63)
               {
                  OSCTUNE=c0;
                 //cx=OSCTUNE
                 CPrint("\r\n New OSCTUNE= ");
                 ByteToStr(OSCTUNE,CRam1);
                 Print(CRam1);
                 CRLF1();
               }
             }
            if ( (   *(Buffer1)=='R'  ) &&(   *(Buffer1+2)=='C'  ) )
            {
              CPrint("\r\n Note R");
              cx= *(Buffer1+1);
              c0=cx-48;
              if (c0<4)
              {
                drow=c0;
                UART1_Write(drow+48);
                cx= *(Buffer1+3);
                c0=cx-48;
               if (c0<4)
               {
                  dcol=c0;
                  CPrint(" C");
                  UART1_Write(dcol+48);
               }
               CRLF1();
              }
            }
         }
           i=drow*4+dcol;
           UART1_Write(' ');
           UART1_Write(*(Sequence+i));
           UART1_Write(' ');
           CPrint(Leds_HT9170B[i]);
           UART1_Write(' ');
           UART1_Write(TAB);
           CPrint(" F1=");
           WordToStr(Frow_value[drow],CRam1);
           Print(CRam1);
           CPrint(" Hz , F2=");
           WordToStr(Fcol_value[dcol],CRam1);
           Print(CRam1);    CPrint(" Hz");
       //  CPrint(" Hz   PA=");
           BDA_periodA = Frow[drow];
       //    LongWordToStr( BDA_periodA,CRam1);
       //    Print(CRam1); CPrint("  PB=");
           BDA_periodB = Fcol[dcol];
       //    LongWordToStr( BDA_periodB,CRam1);
       //    Print(CRam1);

           CRLF1();
           SQA=1;
           make_DTMF_tone();
           SQA=0;
     } while(1);
  }


/*


  Presentation :
 Directory :C:\_MikroC\_MesProjets_MikroC\_18F27K42_Roman_Black_DTMF_Generateur_2022
 MikroC pro 7.60
 Projet :PIC18F27K42_Roman_Black_DTMF_Generateur_2022
 generation de sons DTMF via 8bits PCM output
 Config bit : P18F27K42_Fosc_Interne_Legacy_interrupts_16x4_PLL_64MHz.cfgsch
 FOSC:64.0 MHz
 Eeprom: not used ....
 Source : PIC18F27K42_Roman_Black_DTMF_Generateur_RC2_ou_RC4_2022_1206.c
 18F27K42 UART1 19200 bds
 usage de l'Algorithme de Roman Black
 PWM1 sur RC2 par defaut  si RB1=1 , UART1 : Rx RC7  TX RC6

 Sinewave DTMF generator
 Select the 2 DTMF tones
 Row :   697    770    852    941    OFF
 Col :  1209   1336   1477   1633    OFF

 Init PWM1 at 62500Hz  Choix RB1=0 => PWM  sur sortie  RC4
 FIGURE 23-5: PWM 10-BIT ALIGNMENT page 349
 Etat bit FMT = 1 Alignement  gauche
 Test CCP1 en mode PWM associ  TMR2, pwm 16  224
 pwm1 =  16
 pwm1 =  32
 pwm1 =  48
 pwm1 =  64
 pwm1 =  80
 pwm1 =  96
 pwm1 = 112
 pwm1 = 128
 pwm1 = 144
 pwm1 = 160
 pwm1 = 176
 pwm1 = 192
 pwm1 = 208
 pwm1 = 224

 Init Timer6  3 sec : ............
 Remise FMT bit   Zero !!!
     pour pouvoir reutiliser la fonction MikroC (en 8 bits)
 default starting values for the DTMF drow=0 and dcol=0
 F1=  697 Hz , F2= 1209 Hz

 Generation des 16 codes DTMF  intervalle 1sec
 1 LLLH	 F1=  697 Hz , F2= 1209 Hz   PA=     46775  PB=     81135
 2 LLHL	 F1=  697 Hz , F2= 1336 Hz   PA=     46775  PB=     89657
 3 LLHH	 F1=  697 Hz , F2= 1477 Hz   PA=     46775  PB=     99120
 A HHLH	 F1=  697 Hz , F2= 1633 Hz   PA=     46775  PB=    109589
 4 LHLL	 F1=  770 Hz , F2= 1209 Hz   PA=     51674  PB=     81135
 5 LHLH	 F1=  770 Hz , F2= 1336 Hz   PA=     51674  PB=     89657
 6 LHHL	 F1=  770 Hz , F2= 1477 Hz   PA=     51674  PB=     99120
 B HHHL	 F1=  770 Hz , F2= 1633 Hz   PA=     51674  PB=    109589
 7 LHHH	 F1=  852 Hz , F2= 1209 Hz   PA=     57177  PB=     81135
 8 HLLL	 F1=  852 Hz , F2= 1336 Hz   PA=     57177  PB=     89657
 9 HLLH	 F1=  852 Hz , F2= 1477 Hz   PA=     57177  PB=     99120
 C HHHH	 F1=  852 Hz , F2= 1633 Hz   PA=     57177  PB=    109589
 0 HLHL	 F1=  941 Hz , F2= 1209 Hz   PA=     63149  PB=     81135
 * HLHH	 F1=  941 Hz , F2= 1336 Hz   PA=     63149  PB=     89657
 # HHLL	 F1=  941 Hz , F2= 1477 Hz   PA=     63149  PB=     99120
 D LLLL	 F1=  941 Hz , F2= 1633 Hz   PA=     63149  PB=    109589

 OSCTUNE ajustable via cde Oxx   avec xx 0  62
 code touche 1 par defaut
 modif code via saisie de RxCx  (Range et Colonne de touche)

 1 LLLH 	 F1=  697 Hz , F2= 1209 Hz
 1 LLLH 	 F1=  697 Hz , F2= 1209 Hz
R0C1

 Note R0 C1
 2 LLHL 	 F1=  697 Hz , F2= 1336 Hz
 2 LLHL 	 F1=  697 Hz , F2= 1336 Hz
R0C2

 Note R0 C2
 3 LLHH 	 F1=  697 Hz , F2= 1477 Hz
 3 LLHH 	 F1=  697 Hz , F2= 1477 Hz
R1C0

 Note R1 C0
 4 LHLL 	 F1=  770 Hz , F2= 1209 Hz
 4 LHLL 	 F1=  770 Hz , F2= 1209 Hz
R1C1

 Note R1 C1
 5 LHLH 	 F1=  770 Hz , F2= 1336 Hz
 5 LHLH 	 F1=  770 Hz , F2= 1336 Hz
R1C2

 Note R1 C2
 6 LHHL 	 F1=  770 Hz , F2= 1477 Hz
 6 LHHL 	 F1=  770 Hz , F2= 1477 Hz
R2C0
 6 LHHL 	 F1=  770 Hz , F2= 1477 Hz
R2C0

 Note R2 C0
 7 LHHH 	 F1=  852 Hz , F2= 1209 Hz
 7 LHHH 	 F1=  852 Hz , F2= 1209 Hz
R2C1

 Note R2 C1
 8 HLLL 	 F1=  852 Hz , F2= 1336 Hz
 8 HLLL 	 F1=  852 Hz , F2= 1336 Hz
R3C1

 Note R3 C1
 * HLHH 	 F1=  941 Hz , F2= 1336 Hz
 * HLHH 	 F1=  941 Hz , F2= 1336 Hz
 * HLHH 	 F1=  941 Hz , F2= 1336 Hz
 * HLHH 	 F1=  941 Hz , F2= 1336 Hz
R1C3

 Note R1 C3
 B HHHL 	 F1=  770 Hz , F2= 1633 Hz
 B HHHL 	 F1=  770 Hz , F2= 1633 Hz
 B HHHL 	 F1=  770 Hz , F2= 1633 Hz
 B HHHL 	 F1=  770 Hz , F2= 1633 Hz
 B HHHL 	 F1=  770 Hz , F2= 1633 Hz
 B HHHL 	 F1=  770 Hz , F2= 1633 Hz
 B HHHL 	 F1=  770 Hz , F2= 1633 Hz
 B HHHL 	 F1=  770 Hz , F2= 1633 Hz
R0C3

 Note R0 C3
 A HHLH 	 F1=  697 Hz , F2= 1633 Hz
 A HHLH 	 F1=  697 Hz , F2= 1633 Hz
 A HHLH 	 F1=  697 Hz , F2= 1633 Hz
 A HHLH 	 F1=  697 Hz , F2= 1633 Hz
 A HHLH 	 F1=  697 Hz , F2= 1633 Hz
R2C3

 Note R2 C3
 C HHHH 	 F1=  852 Hz , F2= 1633 Hz
 C HHHH 	 F1=  852 Hz , F2= 1633 Hz
 C HHHH 	 F1=  852 Hz , F2= 1633 Hz
R3C3

 Note R3 C3
 D LLLL 	 F1=  941 Hz , F2= 1633 Hz
 D LLLL 	 F1=  941 Hz , F2= 1633 Hz
 D LLLL 	 F1=  941 Hz , F2= 1633 Hz



0 1 mikroCPIC1618.exe -MSF -DBG -pP18F27K42 -RA -C -O11111014 -fo64 -N"C:\_MikroC\_MesProjets_MikroC\_18F27K42_Roman_Black_DTMF_Generateur_2022\PIC18F27K42_Roman_Black_DTMF_Generator_2022.mcppi" -SP"C:\_MikroC\mikroC PRO for PIC\Defs\" -SP"C:\_MikroC\mikroC PRO for PIC\uses\P18\" -SP"C:\_MikroC\_MesProjets_MikroC\_18F27K42_Roman_Black_DTMF_Generateur_2022\" -IP"C:\_MikroC\mikroC PRO for PIC\uses\P18\" "PIC18F27K42_Roman_Black_DTMF_Generateur_RC2_ou_RC4_2022_1206.c" "__Lib_Math.mcl" "__Lib_MathDouble.mcl" "__Lib_System.mcl" "__Lib_Delays.mcl" "__Lib_CType.mcl" "__Lib_CString.mcl" "__Lib_CStdlib.mcl" "__Lib_Conversions.mcl" "__Lib_Sprintf.mcl" "__Lib_PPS_2xK42.mcl" "__Lib_UART_Remappable_K42_83.mcl" "__Lib_PWM_Remappable_c21b50.mcl"
0 1139 Available RAM: 8170 [bytes], Available ROM: 131072 [bytes]
0 122 Compilation Started P18F27K42.c
7913 123 Compiled Successfully P18F27K42.c
0 122 Compilation Started __Lib_Delays.c
172 123 Compiled Successfully __Lib_Delays.c
0 122 Compilation Started PIC18F27K42_Roman_Black_DTMF_Generateur_RC2_ou_RC4_2022_1206.c
0 1004 interrupt handler (Interrupts at 0x0008) PIC18F27K42_Roman_Black_DTMF_Generateur_RC2_ou_RC4_2022_1206.c
820 123 Compiled Successfully PIC18F27K42_Roman_Black_DTMF_Generateur_RC2_ou_RC4_2022_1206.c
0 127 All files Compiled in 140 ms
0 1144 Used RAM (bytes): 578 (7%)  Free RAM (bytes): 7592 (93%) Used RAM (bytes): 578 (7%)  Free RAM (bytes): 7592 (93%)
0 1144 Used ROM (bytes): 9481 (7%)  Free ROM (bytes): 121591 (93%) Used ROM (bytes): 9481 (7%)  Free ROM (bytes): 121591 (93%)
0 125 Project Linked Successfully PIC18F27K42_Roman_Black_DTMF_Generator_2022.mcppi
0 128 Linked in 94 ms
0 129 Project 'PIC18F27K42_Roman_Black_DTMF_Generator_2022.mcppi' completed: 594 ms
0 103 Finished successfully: 06 dc. 2022, 14:15:29 PIC18F27K42_Roman_Black_DTMF_Generator_2022.mcppi

*/